<?php

class Image_Processor {
	
	var $settings;
	var $imageFile;
	var $image;
	var $metadataIrb = array();
	var $metadataXmp = array();
	
	
	function __construct( $settings = array() )
	{
		ini_set( 'gd.jpeg_ignore_warning', TRUE );

		$this->settings = array(
			'watermarkMargin' => 15,
			'fontPath' => dirname( __FILE__ ) . '/../resources/bitstream-vera/'
		);
		
		if ( ! empty( $settings ) )
		{
			$this->settings = array_merge( $this->settings, $settings );
		}
	}


	function load( $imageFile, $preserveMetadata = FALSE )
	{
		list( $width, $height ) = getimagesize( $imageFile );
		$memoryNeed = $width * $height * 3;
		
		if ( $memoryNeed > $this->_get_available_memory() / 3 )
		{
			ini_set( 'memory_limit', $memoryNeed );
			ini_set( 'se_memory_limit', $memoryNeed );
			
			if ( ini_get( 'memory_limit' ) != $memoryNeed 
					&& ini_get( 'se_memory_limit' ) != $memoryNeed )
			{
				return FALSE;
			}
		}

		$this->imageFile = $imageFile;
		$this->image = imagecreatefromjpeg( $imageFile );

		if ( $preserveMetadata )
		{
			$meta = new Metadata_Manager( $imageFile );
			$this->metadataIrb = $meta->get_irb_data();
			$this->metadataXmp = $meta->get_xmp_data();
		}

		return TRUE;
	}


	function resize_to_height( $height, $autorotate = TRUE, $upsizing = FALSE )
	{
		// skaliert das Bild auf eine bestimmte Höhe

		$imageWidth = imagesx( $this->image );
		$imageHeight = imagesy( $this->image );
		
		if ( $imageHeight < $height && ! $upsizing )
		{
			return; // keine Skalierung erforderlich
		}
		
		
	}

	function resize( $size, $minSize = 0, $autorotate = TRUE, $upsizing = FALSE )
	{
		// skaliert das Bild so, dass die längere Seite $size Pixel lang wird
		// $size wird ggf. erhöht, so dass die kürzere Seite mindestens $minSize lang wird

		$width = imagesx( $this->image );
		$height = imagesy( $this->image );
		$minSize = (int) $minSize > 0 ? (int) $minSize : 0;

		if ( max( $width, $height ) == $size 
				&& ( ! $minSize || min( $width, $height ) >= $minSize ) )
		{
			return; // das Bild hat bereits die richtige Größe
		}

		if ( $width > $height )
		{
			$newWidth = $size;
	  		$newHeight = ceil( $size * $height / $width );
		}
		else
		{
			$newHeight = $size;
	  		$newWidth = ceil( $size * $width / $height );
		}
		
		if ( $minSize && min( $newWidth, $newHeight ) < $minSize )
		{
			if ( $newWidth < $newHeight )
			{
				$newWidth = $minSize;
		  		$newHeight = ceil( $minSize * $height / $width );
			}
			else
			{
				$newHeight = $minSize;
		  		$newWidth = ceil( $minSize * $width / $height );
			}
		}
		
		if ( $newHeight > $height && ! $upsizing )
		{
			return; // Bild ist bereits größer als Zielgröße
		}

		$imageNew = imagecreatetruecolor( $newWidth, $newHeight );
		imagecopyresampled( $imageNew, $this->image, 0, 0, 0, 0, $newWidth, $newHeight, $width, $height );
		
		if ( $autorotate && function_exists( 'exif_read_data' ) )
		{
			$angles = array( 3 => 180, 6 => -90, 8 => 90 );
			$exif = exif_read_data( $this->imageFile );
			
			if ( $exif && isset( $exif['Orientation'] ) && isset( $angles[ $exif['Orientation'] ] ) )
			{
				$imageNew = imagerotate( $imageNew, $angles[ $exif['Orientation'] ], 0 );
			}
		}

		imagedestroy( $this->image );
		$this->image = $imageNew;
	}


	function add_watermark_text( $text, $alignment = 'c', $opacity = 70, $size = 0.7, $color = 'ffffff', $font = 'Vera.ttf', $rotation = 0 )
	{
		// This function is based on source code from
		// http://salman-w.blogspot.com/2008/11/watermark-your-images-with-text-using.html

		$imageWidth = imagesx( $this->image );
		$imageHeight = imagesy( $this->image );
		
		if ( $size <= 0 || $size > 1 )
		{
			$size = 0.7;
		}

		// Interprete the size as a scaling factor for the text box
		// relative to the image width

		$box = $this->_imagettfbbox_fixed( 24, $rotation, $font, $text );
		$size = 24 * $size * $imageWidth / ( $box[2] - $box[1] );
		
		$box = $this->_imagettfbbox_fixed( $size, $rotation, $font, $text );
		$x0 = min( $box[0], $box[2], $box[4], $box[6] ) - $this->settings['watermarkMargin'];
		$x1 = max( $box[0], $box[2], $box[4], $box[6] ) + $this->settings['watermarkMargin'];
		$y0 = min( $box[1], $box[3], $box[5], $box[7] ) - $this->settings['watermarkMargin'];
		$y1 = max( $box[1], $box[3], $box[5], $box[7] ) + $this->settings['watermarkMargin'];
		$boxWidth = abs( $x1 - $x0 );
		$boxHeight = abs( $y1 - $y0 );

		switch ( $alignment )
		{
			case 'nw':
				$bpy = -$y0;
				$bpx = -$x0;
				break;
			case 'n':
				$bpy = -$y0;
				$bpx = $imageWidth / 2 - $boxWidth / 2 - $x0;
				break;
			case 'ne':
				$bpy = -$y0;
				$bpx = $imageWidth - $x1;
				break;
			case 'w':
				$bpy = $imageHeight / 2 - $boxHeight / 2 - $y0;
				$bpx = -$x0;
				break;
			case 'c':
				$bpy = $imageHeight / 2 - $boxHeight / 2 - $y0;
				$bpx = $imageWidth / 2 - $boxWidth / 2 - $x0;
				break;
			case 'e':
				$bpy = $imageHeight / 2 - $boxHeight / 2 - $y0;
				$bpx = $imageWidth - $x1;
				break;
			case 'sw':
				$bpy = $imageHeight - $y1;
				$bpx = -$x0;
				break;
			case 's':
				$bpy = $imageHeight - $y1;
				$bpx = $imageWidth / 2 - $boxWidth / 2 - $x0;
				break;
			case 'se';
				$bpy = $imageHeight - $y1;
				$bpx = $imageWidth - $x1;
				break;
		}

		$alphaColor = imagecolorallocatealpha(
			$this->image,
			hexdec( substr( $color, 0, 2 ) ),
			hexdec( substr( $color, 2, 2 ) ),
			hexdec( substr( $color, 4, 2 ) ),
			127 * ( 100 - $opacity ) / 100
		);

		$shadowColor = imagecolorallocatealpha( $this->image, 0, 0, 0, 127 * ( 100 - $opacity ) / 100 );

		imagettftext( $this->image, $size, $rotation, $bpx + 2, $bpy + 2, $shadowColor, $this->settings['fontPath'] . $font, $text );
		imagettftext( $this->image, $size, $rotation, $bpx, $bpy, $alphaColor, $this->settings['fontPath'] . $font, $text );
	}


	function add_watermark_image( $watermarkFile, $alignment = 'c', $opacity = 70, $scaling = 0.7 )
	{
		// This function is based on source code from
		// http://salman-w.blogspot.com/2008/11/watermark-your-images-with-another.html

		// Watermark Image should be created with the following settings:
		// File Format: PNG-8, non-interlaced
		// Color Reduction: Selective, 256 colors
		// Dithering: Diffusion, 88%
		// Transparency: On, Matte: None
		// Transparency Dither: Diffusion Transparency Dither, 100%

		$imageWidth = imagesx( $this->image );
		$imageHeight = imagesy( $this->image );
		$scaling = max( 0, min( 1, $scaling ) );

		$watermark = imagecreatefrompng( $watermarkFile );
		$watermarkWidth = imagesx( $watermark );
		$watermarkHeight = imagesy( $watermark );

		// Resize watermark if neccessary
		// If the $size parameter is zero, the watermark is placed in its original size
		// Otherwise it is scaled by $scaling (in relation to the narrow side of the original image)
		
		if ( $scaling )
		{
			$imageRatio = $imageWidth / $imageHeight;
			$watermarkRatio = $watermarkWidth / $watermarkHeight;

			if ( $imageRatio < $watermarkRatio )
			{
				$watermarkWidth = $imageWidth * $scaling;
				$watermarkHeight = $watermarkWidth / $watermarkRatio;
			}
			else
			{
				$watermarkHeight = $imageHeight * $scaling;
				$watermarkWidth = $watermarkHeight * $watermarkRatio;
			}

			$newWatermark = imagecreatetruecolor( $watermarkWidth, $watermarkHeight );
			imagealphablending( $newWatermark, FALSE );
			imagesavealpha( $newWatermark, TRUE );
			$transparent = imagecolortransparent( $newWatermark, imagecolorallocatealpha( $newWatermark, 0, 0, 0, 127 ) );
			imagefill( $newWatermark, 0, 0, $transparent );
			imagecopyresized( $newWatermark, $watermark, 0, 0, 0, 0, $watermarkWidth, $watermarkHeight, imagesx( $watermark ), imagesy( $watermark ) );
			imagedestroy( $watermark );
			$watermark = $newWatermark;
		}
		
		switch ( $alignment )
		{
			case 'nw':
				$x = 0;
				$y = 0;
				break;
			case 'n':
				$x = $imageWidth / 2 - $watermarkWidth / 2;
				$y = 0;
				break;
			case 'ne':
				$x = $imageWidth - $watermarkWidth;
				$y = 0;
				break;
			case 'w':
				$x = 0;
				$y = $imageHeight / 2 - $watermarkHeight / 2;
				break;
			case 'c':
				$x = $imageWidth / 2 - $watermarkWidth / 2;
				$y = $imageHeight / 2 - $watermarkHeight / 2;
				break;
			case 'e':
				$x = $imageWidth - $watermarkWidth;
				$y = $imageHeight / 2 - $watermarkHeight / 2;
				break;
			case 'sw':
				$x = 0;
				$y = $imageHeight - $watermarkHeight;
				break;
			case 's':
				$x = $imageWidth / 2 - $watermarkWidth / 2;
				$y = $imageHeight - $watermarkHeight;
				break;
			case 'se':
				$x = $imageWidth - $watermarkWidth;
				$y = $imageHeight - $watermarkHeight;
				break;
			default:
				$x = $imageWidth / 2 - $watermarkWidth / 2;
				$y = $imageHeight / 2 - $watermarkHeight / 2;
		}
		
		imagecopymerge( $this->image, $watermark, $x, $y, 0, 0, $watermarkWidth, $watermarkHeight, $opacity );
		imagedestroy( $watermark );
	}


	function write( $file, $quality = 90 )
	{
		if ( empty( $file ) )
		{
			return FALSE;
		}

		$parts = explode( '/', dirname( $file ) );
		$dir = '';

		foreach ( $parts as $part )
		{
			$dir .= $part . '/';

			if ( ! @is_dir( $dir ) && strlen( $dir ) > 0 )
			{
				@mkdir( $dir );
				@chmod( $dir, 0777 );
			}
		}

		if ( ! imagejpeg( $this->image, $file, $quality ) )
		{
			return FALSE;
		}

		if ( ! empty( $this->metadataIrb ) || ! empty( $this->metadataXmp ) )
		{
			$meta = new Metadata_Manager( $file );
			$meta->put_irb_data( $this->metadataIrb );
			$meta->put_xmp_data( $this->metadataXmp );
			$meta->write( $file );
		}
		
		return TRUE;
	}


	function _imagettfbbox_fixed( $size, $rotation, $font, $text )
	{
		 // Fix for the buggy imagettfbbox implementation in gd library

		$box = imagettfbbox( $size, 0, $this->settings['fontPath'] . $font, $text );
		$angle = deg2rad( $rotation );
		$result = array();

		for ( $i = 0; $i < 7; $i += 2 )
		{
			$result[ $i + 0 ] = round( $box[ $i + 0 ] * cos( $angle ) + $box[ $i + 1 ] * sin( $angle ) );
			$result[ $i + 1 ] = round( $box[ $i + 1 ] * cos( $angle ) - $box[ $i + 0 ] * sin( $angle ) );
		}

		return $result;
	}


	function _get_available_memory()
	{
		$value = trim( ini_get( 'memory_limit' ) );
		
		if ( $value == '-1' )
		{
			$value = '1T';
		}

		$unit = strtolower( substr( $value, -1 ) );
		$value1 = $value ? $value * pow( 1024, strpos( '*kmgt', $unit ) ) : 0;
		$value = trim( ini_get( 'se_memory_limit' ) );

		if ( $value == '-1' )
		{
			$value = '1T';
		}

		$unit = strtolower( substr( $value, -1 ) );
		$value2 = $value ? $value * pow( 1024, strpos( '*kmgt', $unit ) ) : 0;
		return max( $value1, $value2 );
	}

}

// end of file
